#import <Foundation/Foundation.h>
#import "OCCInvocation.h"

typedef void (^OCCConnectionHandler)(Class serviceClass, NSError *error);

@interface NSURL (OCCPublicObject)

// Creates URLs appropriate for connecting with the service of a cloud application
// depending on several parameters. You can pass the result of this method to
// +connectToURL:protocol:.
// Parameters:
// cloudApplicationName: The name of your cloud application as specified at https://connect.objective-cloud.com
//                       This parameter can only be nil if forWorkingLocally is YES.
// serviceName: The name of the class you wanna talk to. Cannot be nil.
// forWorkingLocally: If set to YES then you will get back a URL targeting localhost.
//                    If set to NO then you will get back a URL targeting your live system at
//                    obcl.io (using https instead of http).
//+ (instancetype)URLWithCloudApplicationName:(NSString *)cloudApplicationName
//                                serviceName:(NSString *)serviceName
//                          forWorkingLocally:(BOOL)forWorkingLocally;
@end

@interface OCCService : NSProxy

#pragma mark - Connecting
/**
 *	Sets the remote host
 *
 *  @param host
 *
 *  @note Sets the remote host used for linking wrapper classes to services. The remote host is always used over https. The default is @"obcl.io".
 */
+ (void)setRemoteHost:(NSString*)host;

/**
 *	Sets the local host
 *
 *  @param host
 *
 *  @note Sets the local host used for linking wrapper classes to services. The local host is always used over http. The default is "localhost:10000"
 */
+ (void)setLocalHost:(NSString*)host;

/**
 *	Switches to remote
 *
 *	@param worksRemotly
 */
+ (void)setUsesRemoteHost:(BOOL)worksRemotly;

+ (BOOL)linkToCloudApp:(NSString*)cloudApp protocol:(Protocol*)protocol URLSession:(NSURLSession *)URLSession handler:(OCCConnectionHandler)handler;
+ (BOOL)linkToCloudApp:(NSString*)cloudApp protocol:(Protocol*)protocol handler:(OCCConnectionHandler)handler;


+ (NSString*)completeHostName;

#pragma Asynchronous Invocation
+ (id)invokeWithHandler:(OCCInvocationResponseHandler)handler;
+ (id)invokeAndDispatchOnMainQueue:(BOOL)dispatchesOnMainQueue handler:(OCCInvocationResponseHandler)handler;
+ (id)invokeWithReturnHandler:(OCCInvocationReturnHandler)returnHandler errorHandler:(OCCInvocationErrorHandler)errorHandler;

#pragma mark - Creating
- (instancetype)init;
+ (instancetype)new;
@end

@interface NSObject (OCCCloudError)
- (BOOL)isObjectiveCloudError;
- (BOOL)isObjectiveCloudErrorTheEndgeilMethodOfChris;
- (NSError*)objectiveCloudError;
@end